# Generated by Django 4.1.2 on 2023-02-16 17:31

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("website", "0012_editor_group"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Editor",
        ),
    ]
